﻿using ff14bot;
using ff14bot.Managers;
using ff14bot.Objects;
using Reborn_Healer.Profile.Combat;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Reborn_Healer.Tasks.Combat
{
    /// <summary>
    /// Responsible for taking a game state and evaluating the next, logical option given the profile specified.
    /// </summary>
    public class SkillGenerator
    {
        /// <summary>
        ///  A structure representing basic skill info
        /// </summary>
        public class SkillInfo
        {
            public Character Target { get; set; }

            public SkillEntry SkillEntry { get; set; }

            public SkillInfo(Character target, SkillEntry skillEntry)
            {
                Target = target;
                SkillEntry = skillEntry;
            }
        }

        /// <summary>
        /// Combat profile to be tested against during the generation phase
        /// </summary>
        public CombatProfile SkillProfile { get; set; }

        public SkillGenerator(CombatProfile profile)
        {
            SkillProfile = profile;
        }

        /// <summary>
        /// Gets the next <see cref="SkillEntry"/> that is applicable to the game state, given the current profile.
        /// </summary>
        /// <returns>The next valid skill is returned if possible, otherwise null is returned indicating there is nothing to perform.</returns>
        public SkillInfo GetNextEntry()
        {
            // Loop over each entry, we need to check each one for possibilities
            foreach (var skillEntry in SkillProfile.Entries)
            {

                var isUsable = IsEntryUsable(skillEntry);

                if (isUsable != null && Actionmanager.CanCast(isUsable.SkillEntry.SkillId, isUsable.Target))
                    return isUsable;

            }

            return null;
        }

        /// <summary>
        /// Determines whether a given entry is valid for use given the condition it is bound with.
        /// </summary>
        /// <param name="entry"></param>
        private SkillInfo IsEntryUsable(SkillEntry entry)
        {
            var context = ScopeGenerator.GetScopeBattleCharacter(Core.Player, Scope.Ally);
            var passingAllies = context.ToList();

            // Remove yourself, and then re-add. This will make the ally list exactly '1' when playing solo
            passingAllies.RemoveAll(x => x.IsMe);

            foreach (var ally in context)
            {
                var pass = true;
                foreach (var condition in entry.Conditions)
                {
                    if (condition.ConditionScope != Scope.Ally)
                        continue;

                    pass &= condition.Evaluate(ally);
                }

                if (!pass)
                {
                    if (passingAllies.Contains(ally))
                        passingAllies.Remove(ally);
                }
            } // gets a list of allies who pass the initial tests

            bool okay = true;

            // Now, we can test everyone else whom is a singular case (we hope)
            foreach (var condition in entry.Conditions)
            {
                if (condition.ConditionScope == Scope.Ally)
                    continue;

                var conditionContext = ScopeGenerator.GetScopeBattleCharacter(Core.Player, condition.ConditionScope);

                foreach (var BattleCharacter in conditionContext)
                {
                    if (BattleCharacter == null)
                    {
                        okay = false;
                        break;
                    }
                    okay &= condition.Evaluate(BattleCharacter);
                }
            }

            // Everything went according to plan
            if (okay && (passingAllies.Count > 0 || (!PartyManager.IsInParty || PartyManager.VisibleMembers.Count() == 1) ))
            {
                SkillInfo info;
                switch (entry.Target)
                {
                    case Scope.Self:
                        info = new SkillInfo(Core.Player, entry);
                        break;

                    case Scope.Ally:
                        // Just grab the first one for now... u eve
                        //TODO: Implement a priority system

                        // This skill cannot be fired
                        if (!PartyManager.IsInParty || PartyManager.VisibleMembers.Count() == 1)
                            return null;

                        info = new SkillInfo(passingAllies[0], entry);
                        break;

                    case Scope.Target:
                        var n = Core.Player.CurrentTarget.ObjectId;
                        var x =
                            GameObjectManager.GetObjectsOfType<BattleCharacter>().FirstOrDefault(y => y.ObjectId == n);

                        if (x == null)
                            return null;

                        info = new SkillInfo(x, entry);
                        break;

                    case Scope.Pet:
                        throw new Exception("Pet does not make sense as a target. It's undefined.");
                    default:
                        throw new NotImplementedException("The given type " + entry.Target + " is not a valid target type.");
                }

                return info;
            }

            return null;
        }

        /// <summary>
        /// Given a list of allies, uses the SkillProfile CombatPriority to determine whom should be selected.
        /// </summary>
        /// <param name="allies">The list of allies to filter and select</param>
        /// <returns></returns>
        private BattleCharacter GetAllyByPriority(List<BattleCharacter> allies)
        {
            return null;
        }
    }
}