﻿using ff14bot.Managers;
using Reborn_Healer.Managers;
using Reborn_Healer.Profile.Combat;
using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace Reborn_Healer.UI.Combat
{
    public partial class CombatProfileEditor : UserControl
    {
        #region Variables

        private List<CombatProfile> _combatProfiles;
        private CombatProfile _currentProfile;

        #endregion Variables

        public CombatProfileEditor()
        {
            InitializeComponent();

            // We need to ensure we have at least something selected and something is available to us
            NewOrExistingProfile();

            //Grab the players current skills from current job.
            SetupSkills();
        }

        #region Setup Current Job skills in our list.

        /// <summary>
        /// Pull current skill list that player has on current job.
        /// </summary>
        private void SetupSkills()
        {
            foreach (var spellsByJob in Actionmanager.CurrentActions)
            {
                listSkills.Items.Add(spellsByJob.Value.Name);
            }
        }

        #endregion Setup Current Job skills in our list.

        #region Let's fetch our profiles and make sure we have a new default clean profile.

        private void NewOrExistingProfile()
        {
            // Fetch our profiles
            _combatProfiles = (ModeProfileManager.Instance.GetProfiles<CombatProfile>());

            switch (_combatProfiles.Count > 0)
            {
                case false:

                    var newprofile = new CombatProfile("New Profile");
                    _combatProfiles.Add(newprofile);

                    foreach (CombatProfile cb in _combatProfiles)
                    {
                        cbxCurrentProfile.Items.Add(cb);
                    }

                    // Set the current index
                    cbxCurrentProfile.SelectedIndex = 0;
                    break;

                case true:
                    if (!_combatProfiles.Contains(new CombatProfile("New Profile")))
                    {
                        var existprofile = new CombatProfile("New Profile");
                        _combatProfiles.Add(existprofile);
                    }

                    foreach (CombatProfile cb in _combatProfiles)
                    {
                        cbxCurrentProfile.Items.Add(cb);
                    }

                    // Set the current index
                    cbxCurrentProfile.SelectedIndex = 0;
                    break;
            }
        }

        #endregion Let's fetch our profiles and make sure we have a new default clean profile.

        #region Profiles Selected Index Changed

        /// <summary>
        /// Changes profiles to currently selected index. Clears list and readds to items based on profile selected.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cbxCurrentProfile_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Set our current profile
            _currentProfile = (CombatProfile)cbxCurrentProfile.SelectedItem;

            // Set our textbox to currentprofile incase user wants to clone and modify off of another profile.
            textName.Text = _currentProfile.Name;

            //Clear our profile skills list so we can only have current profile skills.
            lstbxSkillsProfile.Items.Clear();

            //Add our new currently selected profile skills.
            foreach (var t in _currentProfile.Entries)
            {
                lstbxSkillsProfile.Items.Add(DataManager.SpellCache[t.SkillId].Name);
            }
        }

        #endregion Profiles Selected Index Changed

        #region Save Profiles

        /// <summary>
        /// Attempt to save our profile to current profile selected. If none selected then Save as New Profile.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSaveProfile_Click(object sender, EventArgs e)
        {
            // Attempt to save
            if (_currentProfile != null && textName.Text == _currentProfile.Name)
            {
                if (_currentProfile.Name == "New Profile")
                {
                    MessageBox.Show("Please select a profile name other then New Profile", "Change Profile Name", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return;
                }

                ModeProfileManager.Instance.SaveProfile<CombatProfile>(_currentProfile);
            }
            else
            {
                if (textName.Text != null && !string.IsNullOrWhiteSpace(textName.Text))
                {
                    //Start new combat profile based on text in textbox
                    var profile = new CombatProfile(textName.Text);

                    //just incase user has skills already in list or is clonin and existing list to a new profile
                    //add them to new profile.
                    profile.Entries = _currentProfile.Entries;

                    _combatProfiles.Add(profile);
                    _currentProfile = profile;

                    //Add profile to list and select it.
                    cbxCurrentProfile.Items.Add(_currentProfile);
                    cbxCurrentProfile.SelectedItem = _currentProfile;

                    ModeProfileManager.Instance.SaveProfile<CombatProfile>(_currentProfile);
                }
            }
        }

        #endregion Save Profiles

        #region Controls adding skills to our list boxes.

        /// <summary>
        /// When user selects a skill it will add it to profile list.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listSkills_MouseClick(object sender, MouseEventArgs e)
        {
            if (e.Button == System.Windows.Forms.MouseButtons.Left)
            {
                ListBox lb = sender as ListBox;

                lstbxSkillsProfile.Items.Add(lb.SelectedItem);

                _currentProfile.CreateSkillEntry(Actionmanager.CurrentActions[lb.SelectedItem.ToString()].Id);
            }
        }

        /// <summary>
        /// When user selects a skill to edit, open a new skill editor for that skill.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void lstbxSkillsProfile_MouseClick(object sender, MouseEventArgs e)
        {
            if (e.Button == System.Windows.Forms.MouseButtons.Left)
            {
                ListBox lb = sender as ListBox;

                var entry = _currentProfile.GetSkillEntry(DataManager.SpellCache[_currentProfile.Entries[lb.SelectedIndex].SkillId].Id, lb.SelectedIndex);

                var skillForm = new SkillEditor(entry);
                skillForm.ShowDialog();
            }
        }

        #endregion Controls adding skills to our list boxes.

        #region Menu Event Handler

        /// <summary>
        /// Menu click event for context menu removal of skills in profile.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuItemOnClick(object sender, EventArgs e)
        {
            var menuName = (MenuItem)sender;

            switch (menuName.Text)
            {
                case "Remove":
                    _currentProfile.RemoveEntryByIdIndex(lstbxSkillsProfile.SelectedIndex);
                    lstbxSkillsProfile.Items.RemoveAt(lstbxSkillsProfile.SelectedIndex);
                    break;

                case "Position Up":
                    _currentProfile.MoveSpellPosition(DataManager.SpellCache[_currentProfile.Entries[lstbxSkillsProfile.SelectedIndex].SkillId].Id
                                                      , true, lstbxSkillsProfile.SelectedIndex);

                    lstbxSkillsProfile.Items.Clear();

                    foreach (var item in _currentProfile.Entries)
                        lstbxSkillsProfile.Items.Add(DataManager.SpellCache[item.SkillId].Name);
                    break;

                case "Position Down":
                    _currentProfile.MoveSpellPosition(DataManager.SpellCache[_currentProfile.Entries[lstbxSkillsProfile.SelectedIndex].SkillId].Id
                                                      , false, lstbxSkillsProfile.SelectedIndex);

                    lstbxSkillsProfile.Items.Clear();

                    foreach (var item in _currentProfile.Entries)
                        lstbxSkillsProfile.Items.Add(DataManager.SpellCache[item.SkillId].Name);
                    break;

                default:
                    throw new Exception("No such menu exists. Did you click a menu?");
            }
        }

        #endregion Menu Event Handler

        #region ReSetup our current job skills.

        /// <summary>
        /// Refreshes current job skills incase of job change.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRefreshSkills_Click(object sender, EventArgs e)
        {
            listSkills.Items.Clear();
            SetupSkills();
        }

        #endregion ReSetup our current job skills.

        #region MouseDown Event.

        /// <summary>
        /// Lets us use the right mouse button to select skills in profile to have removed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void lstbxSkillsProfile_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == System.Windows.Forms.MouseButtons.Right)
            {
                lstbxSkillsProfile.SelectedIndex = lstbxSkillsProfile.IndexFromPoint(e.X, e.Y);

                var menuStrip = new ContextMenu();

                lstbxSkillsProfile.ContextMenu = menuStrip;

                var removeitem = new MenuItem("Remove");
                removeitem.Click += MenuItemOnClick;

                var slotChangeUp = new MenuItem("Position Up");
                slotChangeUp.Click += MenuItemOnClick;

                var slotChangeDown = new MenuItem("Position Down");
                slotChangeDown.Click += MenuItemOnClick;

                menuStrip.MenuItems.Add(removeitem);
                menuStrip.MenuItems.Add(slotChangeUp);
                menuStrip.MenuItems.Add(slotChangeDown);
                lstbxSkillsProfile.ContextMenu.Show(lstbxSkillsProfile, e.Location);
            }
        }

        #endregion MouseDown Event.
    }
}