﻿using Reborn_Healer.Profile.Combat;
using Reborn_Healer.Profile.Combat.Conditions;
using Reborn_Healer.Tasks.Combat;
using System;
using System.Reflection;
using System.Windows.Forms;

namespace Reborn_Healer.UI.Combat
{
    /// <summary>
    /// A skill editor is used to edit the conditions and skill prepositions.
    /// </summary>
    public partial class SkillEditor : Form
    {
        private SkillEntry _skillEntry;

        public SkillEditor(SkillEntry skillEntry)
        {
            _skillEntry = skillEntry;
            InitializeComponent();
            cbxTargetScope.DataSource = Enum.GetValues(typeof(Scope));
            PopulateTreeView();
            RefreshNodeInfo();
        }

        public void RefreshNodeInfo()
        {
            cbxTargetScope.SelectedItem = _skillEntry.Target;

            foreach (var condition in _skillEntry.Conditions)
            {
                switch (condition.GetType().Name)
                {
                    default:
                        throw new NotImplementedException();
                    case "ValueCondition":
                        trvConditions.Nodes["ValueCondition"].Nodes.Add(condition.Name).Tag = condition;
                        break;

                    case "BuffCondition":
                        trvConditions.Nodes["BuffCondition"].Nodes.Add(condition.Name).Tag = condition;
                        break;
                }
            }
        }

        /// <summary>
        /// Populates our tree view list with reflection from ICondition Inheritable class.
        /// Also populates our right click menu based on each individial inherited class.
        /// </summary>
        public void PopulateTreeView()
        {
            foreach (Type type in Assembly.GetExecutingAssembly().GetTypes())
            {
                if (type.IsSubclassOf(typeof(Condition)))
                {
                    // We'll store the type data it's used for on the actual node
                    var node = new TreeNode(type.Name) { Tag = type, Name = type.Name };

                    trvConditions.Nodes.Add(node);

                    // Find the type that has the property catch
                    foreach (Type c in Assembly.GetExecutingAssembly().GetTypes())
                    {
                        var attr = c.GetCustomAttribute<ConditionDescriptionAttribute>();
                        // Ensure the types are integrally equal (the == should be overloaded, this should be okay)
                        if (attr != null && type == attr.ConditionMapping)
                        {
                            // We have the attribute which is good, we can now proceed to map this according
                            var strip = new ContextMenu();

                            // Now, we'll proceed to attach this onto the node
                            node.ContextMenu = strip;

                            // Fetch the names from our listings
                            foreach (var name in Enum.GetValues(c))
                            {
                                var item = new MenuItem(Enum.GetName(c, name)) { Tag = name };
                                item.Click += MenuItemOnClick;
                                strip.MenuItems.Add(item);
                            }
                        }
                    }
                }
            }
        }

        private void MenuItemOnClick(object sender, EventArgs e)
        {
            var menuItem = (MenuItem)sender;
            var node = new TreeNode(menuItem.Text);
            Condition condition;

            switch (((Type)trvConditions.SelectedNode.Tag).Name.ToString())
            {
                case "BuffCondition":
                    condition = new BuffCondition();
                    break;

                case "ValueCondition":
                    condition = new ValueCondition((ValueCondition.ValueOptions)menuItem.Tag);
                    break;

                default:
                    throw new NotImplementedException();
            }

            _skillEntry.Conditions.Add(condition);

            // Add the node
            node.Tag = condition;
            node.Text = condition.Name;
            trvConditions.SelectedNode.Nodes.Add(node);
            trvConditions.SelectedNode = node;
            propertySkill.SelectedObject = node.Tag;
        }

        /// <summary>
        /// Which node was right clicked.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void trvConditions_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            trvConditions.SelectedNode = e.Node;

            //select node and display information.
            if (e.Button == System.Windows.Forms.MouseButtons.Left)
                if (e.Node.Tag is Condition)
                    propertySkill.SelectedObject = e.Node.Tag;

            //Show only if the right mouse button was clicked.
            if (e.Button == System.Windows.Forms.MouseButtons.Right)
                if (trvConditions.SelectedNode.Parent == e.Node)
                    trvConditions.SelectedNode.ContextMenu.Show(trvConditions, e.Location);
        }

        /// <summary>
        ///
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cbxTargetScope_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (this.Created)
            {
                _skillEntry.Target = (Scope)((ComboBox)sender).SelectedItem;
            }
        }
    }
}